
/* 
 * File   : SyncML.c
 * Author : Jasper Geurtsen
 * Desc.  : 
 *      
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *      
 */

#include "Debug.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <signal.h>

#define SYNCML_BACKOFF_TIME (30)  // seconds
#define SYNCML_KILL_TIME    (60)  // seconds

int main( int argc, char** argv )
{
  int pid = 0;
  int returnValue = 0;
  time_t currentTime;
  int secondsElapsed = 0;
  struct stat buf;

  if( argc < 5 || *argv[1] == '?')
  {
    printf( "Usage:\n%s MACAddress syncMLClientChannelNumber syncType syncmlVersion phonebookFilename\n", argv[ 0 ] );
    printf( "example:\n%s 00:11:9F:AE:AF:37 11 syncml 1.1 /var/run/phonebook.dat \n", argv[ 0 ] );
    exit( 1 );
  }

  DBG("SyncML is called\n");
     
  int i = 0;
  for( i = 0; i < argc; i++)
  {
      DBG("------- SyncML argc:%d = %s\n", i, argv[i]);      
  }
  
  // 1) Check if the sync is killed already, if so exit    
  pid = getpid();
  DBG("The PID is %d\n", pid);  
  char killFileName[128];
  sprintf( killFileName, "/var/run/killpb%d.syn", pid);  
  returnValue = stat(killFileName, &buf);
  if( returnValue == 0)
  {
     DBG("------- syncml exiting before starting \n");
     return 0;
  }
  
  // 2) check if there are kills pending
  // If so wait for them to exit before continue.
  // If those pending kills are pending for more then SYNCML_KILL_TIME
  // kill them because some thing is wrong.
  int checkKillFiles = 1;
  while(checkKillFiles)
  {
    struct dirent **namelist;
    int numberOfFiles = scandir("/var/run/", &namelist, 0, alphasort);
    if (numberOfFiles < 0)
    {
      DBG("No pending kills, (no killpb* file)\n");
      checkKillFiles = 0;
    }
    else 
    {
      checkKillFiles = 0;
      while(numberOfFiles--)
      {
        // DBG("PID:%d Filename %s\n", pid, namelist[numberOfFiles]->d_name);
        if (strncmp(namelist[numberOfFiles]->d_name, "killpb", sizeof(char)*6) == 0)
        {
          DBG("PID:%d Filename %s\n", pid, namelist[numberOfFiles]->d_name);
          
          // There is a pending kill.
          checkKillFiles = 1;
          
          sprintf( killFileName, "/var/run/%s", namelist[numberOfFiles]->d_name);
          returnValue = stat(killFileName, &buf);  
          if( returnValue == 0)
          {
            currentTime = time(NULL);
            if( currentTime > buf.st_mtime)
            {              
              secondsElapsed = currentTime - buf.st_mtime;      
              if( secondsElapsed > SYNCML_KILL_TIME )
              {
                // application is not cancelled by it self after SYNCML_KILL_TIME
                // then we kill it, there is something wrong.
                // get PID from file
                FILE* fp = fopen(killFileName, "r");
                if( fp != NULL)
                {
                  char buffer[8];
                  size_t numberOfElements = fread((char*)buffer,1,8,fp);
                  DBG("!!! The kill PID is %s\n", buffer);
                  int pidNumber = atoi(buffer);
                  DBG("!!! The kill PID is %d\n", pidNumber);                  
                  fclose(fp);
                  kill(pidNumber, SIGTERM);
                  remove(killFileName);
                }
              }
              else
              {
                // check if it this process
                sprintf( killFileName, "/var/run/killpb%d.syn", pid);
                returnValue = stat(killFileName, &buf);  
                if( returnValue == 0)
                {    
                   DBG("PID:%d syncml exiting in kill check loop\n", pid);
                   remove(killFileName);
                   free(namelist[numberOfFiles]);
                   while(numberOfFiles--)
                   {
                     free(namelist[numberOfFiles]);
                   }
                   free(namelist);
                   return 0;
                }               
              }                
            }            
          }
          else
          {
            DBG("file stat not oke: %s\n", killFileName);
          }
        }
        free(namelist[numberOfFiles]);
      }
      free(namelist);
      sleep(2);
    }
  }

  // only sleep if we already did a sync.   
  // struct stat buf;
  returnValue = stat("/var/run/cancelledpb.syn", &buf);
  if( returnValue == 0)
  {    
    currentTime = time(NULL);
    if( currentTime > buf.st_mtime)
    {
      secondsElapsed = currentTime - buf.st_mtime;      
      if( secondsElapsed < SYNCML_BACKOFF_TIME )
      {
        DBG("syncml starting waiting %d, difference is %d\n", (SYNCML_BACKOFF_TIME - secondsElapsed), secondsElapsed);
        sleep(SYNCML_BACKOFF_TIME - secondsElapsed);
      }
      else
      {
        DBG("syncml starting NO Sleep file older then %d seconds, difference is %d\n", SYNCML_BACKOFF_TIME, secondsElapsed);
      }
    }
  }
  else
  {
    DBG("NO /var/run/cancelledpb.syn file found\n");
  }

  // Just before we start, check if the app. cancelled the sync.
  sprintf( killFileName, "/var/run/killpb%d.syn", pid);
  returnValue = stat(killFileName, &buf);  
  if( returnValue == 0)
  {
     DBG("PID:%d Syncml exiting just before starting \n", pid);
     remove(killFileName);
     return 0;
  }

  // argv1 = the MAC address of the bluetooth-device (phone)
  // argv2 = the channel of the SyncMLClient
  // argv4 = syncml the version
  // argv5 = filename
  
  _requestSync(argv[1], argv[2], argv[4], argv[5]);  
  
  DBG("PID Syncml finished\n", pid);
  
  return 0;
}
