#include <string.h>
#include <stdio.h>
#include "VCardPhoneNumbers.h"

// #define DEBUG
#include "Debug.h"

#define SUPPORT_UTF8
#define SUPPORT_NAME_SWAP // e.g.:   de Wit Jurgen  ->  Jurgen de Wit

CVCardPhoneNumbers::CVCardPhoneNumbers()
{
  init(NULL);
}

CVCardPhoneNumbers::~CVCardPhoneNumbers()
{  
}

void CVCardPhoneNumbers::init( char *phonebookFilename)
{
  int i = 0;
    
  for( i=0; i < VCARD_MAXNAME_LEN; i++)
  {
    iName[i] = '\0';
  }
  
  if( phonebookFilename != NULL)
  {
    strcpy( iFilename, phonebookFilename);
    DBG("Filename %s\n", iFilename);
  } 
  else  
  {
    iFilename[0] = '\0';
  }
}

char* CVCardPhoneNumbers::getName()
{
  if( iName[0] == '\0' )
    return NULL;
  else
    return iName;
}

char* CVCardPhoneNumbers::setName( char *aData)
{    
  int indexEndOfName = 0;
  int startIndexName = 3;
  size_t stringLength = 0;
  size_t nameLength = 0;
  char tempName[VCARD_MAXNAME_LEN*2];

  if( aData[1] == 'N' && (aData[2] == ':' || aData[2] == ';') )
  {  
    DBG( "There is a Name in vcard.\n");
    stringLength = strlen( aData);

  #ifndef SUPPORT_UTF8
  	// TTA----Fixed special case, when the string is cut off. ex:"=E6=B8=\r\r\n=AC"
    for( indexEndOfName = 1; indexEndOfName < stringLength; indexEndOfName++)
    {         
      if( aData[indexEndOfName] == '\n')
        break;
    }
  #else
    indexEndOfName = stringLength;
  #endif //SUPPORT_UTF8

    if( aData[startIndexName-1] != ':')
    {
      // find the :
      for(; startIndexName < stringLength; startIndexName++)
      {      
        if( aData[startIndexName] == ':')
         break;
      }
      startIndexName++;
    }

    for(; startIndexName < stringLength; startIndexName++)
    {
      if( aData[startIndexName] != ';')   // start index after the starting ;
        break;
    }

    indexEndOfName = indexEndOfName - startIndexName;
    
    // We got the string start and end of the string    
    vcard_encoding_type encodingType = getEncodingType(aData);
    if( encodingType == VCARD_ENCODING_QUOTED_PRINTABLE)
    {
      if( indexEndOfName > ((VCARD_MAXNAME_LEN*2)-1))
        indexEndOfName = (VCARD_MAXNAME_LEN*2)-1;

      for(int x = 0; x < VCARD_MAXNAME_LEN*2; x++) tempName[x] = '\0';
          
      strncpy( tempName, &aData[startIndexName], indexEndOfName);
      convertQPToAscii(encodingType, tempName);
      indexEndOfName = strlen(tempName);
      
      if( indexEndOfName > (VCARD_MAXNAME_LEN-1))
        indexEndOfName = VCARD_MAXNAME_LEN-1;
      
      strncpy( iName, tempName, indexEndOfName);      
    }
    else
    {
      if( indexEndOfName > (VCARD_MAXNAME_LEN-1))
        indexEndOfName = VCARD_MAXNAME_LEN-1;
      strncpy( iName, &aData[startIndexName], indexEndOfName);
    }
    
    nameLength = strlen( iName);

    for(int n = 0; n < nameLength; n++)
    {
      if( iName[n] == '\r')
        iName[n] = '\0';     // remove the carriage return
      if( iName[n] == '\f')
        iName[n] = '\0';     // remove the formfeed   
#ifndef SUPPORT_NAME_SWAP  
      if( iName[n] == ';')   // remove all the ; replace them by <space>            
        iName[n] = ' ';
#endif
    }

    nameLength = strlen( iName);
    // remove the spaces at the end
    for(int n = nameLength-1; n > 0; n--)
    {
      if( iName[n] != ' ') 
      {       
        break;
      }
      else
      {
        iName[n] = '\0';
      }
    }
#ifdef SUPPORT_NAME_SWAP  
    strncpy (tempName, iName, VCARD_MAXNAME_LEN);
    char * p = strchr (tempName, ';' );
    if (p)
    {
      int n;
      memset (iName, 0, VCARD_MAXNAME_LEN );
      
      for(n = 1; n < 10; n++)
      {
        /* Sometimes a lot of ;;; in the name, remove them*/
        if( p[n] != ';') break;
      }
      strncpy (iName, p+n, VCARD_MAXNAME_LEN );
      char * p2 = strchr (iName, ';' );
      if (p2)
      {
        *p2 = 0;
      }
      strncat (iName, " ", 1);
      strncat (iName, tempName, p-tempName);
    }
#endif
  }
  else
  {
    DBG( "Name not found in vcard\n");
  }
}

vcard_encoding_type CVCardPhoneNumbers::getEncodingType(char *aData)
{
  vcard_encoding_type returnValue = VCARD_ENCODING_UNKNOWN;
  char *result = NULL;
  
  result = strstr( aData, "ENCODING=QUOTED-PRINTABLE");  
  if( result != NULL)
  {
    // e.g.:N;CHARSET=UTF-8;ENCODING=QUOTED-PRINTABLE:S=C2=A0;What
    DBG("This is a QUOTED_PRINTABLE encoded vcard\n");
    returnValue = VCARD_ENCODING_QUOTED_PRINTABLE;
  }
  else
  {
    result = strstr( aData, "CHARSET=UTF-8");
    if( result != NULL) 
    {
      DBG("This is a CHARSET=UTF-8 encoded vcard\n");
      returnValue = VCARD_ENCODING_CHARSET_UTF_8;
    }
    else
    {
      DBG("This is an unknown encoded vcard\n");
      returnValue = VCARD_ENCODING_UNKNOWN;
    }
  }

  return returnValue;
}

#ifdef SUPPORT_UTF8
// TTA----Make a function for decode QUOTED_PRINTABLE, we must decode 2 character from number, if we found '='
#define CHAR2INT(x)      (((x)<='9')?(x)-'0':(((x)>='A')?((x)-'A'+10):(-1)))
int DecodeFromQP(unsigned char &integer, char *s)
{
  int i=0;
  int temp;

  do{
    temp = *(s + i++);
    temp = CHAR2INT(temp);
  }while(temp < 0 || temp > 15);
  integer = temp << 4;
  do{
    temp = *(s + i++);
    temp = CHAR2INT(temp);
  }while(temp < 0 || temp > 15);
  integer += temp;
  return i;
}
#endif //SUPPORT_UTF8

void CVCardPhoneNumbers::convertQPToAscii(vcard_encoding_type aType, char *aData)
{
  // remove all =XX, when itn't UTF-8. expect for  
  // '=3D' is '=' 
  // '=A0' is ' ' (space)
  // '=0D' is '\r'
  // '=0A' is '\n'
  // '=C2' replace with nothing
  // '=else' replace with nothing
    
  if( aType == VCARD_ENCODING_QUOTED_PRINTABLE)
  {
    size_t strlength = strlen( aData);
    for(int i = 0;i < strlength;i++)
    {
      // when = then always a 2 hexdec. number behind it
      if( aData[i] == '=')
      {
      #ifdef SUPPORT_UTF8
      	// TTA----There are encode UTF-8 from QUOTED_PRINTABLE.
        unsigned char c[3];
        bool          is_utf8=false;
        int           n = i;
        int           len;
        
        do{
          if(n > strlength-6)
            break;
          n += DecodeFromQP(c[0], &aData[++n]);
          if(c[0] < 128)
            break;
          if ((c[0] & 0xE0) != 0xC0 && (c[0] & 0xF0) != 0xE0)
            break;
          while(aData[n] != '=' && (aData[n] == 0x0D || aData[n] == 0x0A) && n < strlength)
            n++;
          if(n == strlength || aData[n] != '=')
            break;
          n += DecodeFromQP(c[1], &aData[++n]);
          if ((c[1] & 0xC0) != 0x80)
            break;
          if ((c[0] & 0xE0) == 0xC0 )
          {
            aData[i++] = c[0];
            aData[i++] = c[1];
            DBG("UTF-8: %02x%02x\n",c[0],c[1]);
          }
          else
          {
            while(aData[n] != '=' && (aData[n] == 0x0D || aData[n] == 0x0A) && n < strlength)
              n++;
            if(n == strlength || aData[n] != '=')
              break;
            n += DecodeFromQP(c[2], &aData[++n]);
            if ((c[2] & 0xC0) != 0x80 || (c[0] & 0xF0) != 0xE0)
              break;
            aData[i++] = c[0];
            aData[i++] = c[1];
            aData[i++] = c[2];
            DBG("UTF-8: %02x%02x%02x\n",c[0],c[1],c[2]);
          }
          len = n - i;
          strlength -= len;
          for(n = i; n < strlength; n++)
            aData[n] = aData[n+len];
          aData[strlength] = '\0';
          i--;
          is_utf8 = true;
        }while(0);
        if(is_utf8 != true)
      #endif //SUPPORT_UTF8
        {
          if( aData[i+1] == '3' && aData[i+2] == 'D')
          {
            aData[i] = '=';   // replace =3D with =
          }
          else if( aData[i+1] == 'A' && aData[i+2] == '0')
          {
            aData[i] = ' ';   // replace =A0 with ' ' space
          }
          else if( aData[i+1] == '0' && aData[i+2] == 'D')
          {
            aData[i] = '\r';  // replace =0D with '\r'
          }
          else if( aData[i+1] == '0' && aData[i+2] == 'A')
          {
            aData[i] = '\n';  // replace =0A with '\n'
          }
          if( (aData[i+1] == '3' && aData[i+2] == 'D') ||
              (aData[i+1] == 'A' && aData[i+2] == '0') ||
              (aData[i+1] == '0' && aData[i+2] == 'D') ||
              (aData[i+1] == '0' && aData[i+2] == 'A'))
          {
            // shift the everything 2 parts left from here
            strlength -= 2;
            for(int n = i+1; n < strlength; n++)
            {
              aData[n] = aData[n+2];
            }          
            aData[strlength] = '\0';
          }
          else
          {
            // remove =C2 and everything else
            strlength -= 3;
            for(int n = i; n < strlength; n++)
            {
              aData[n] = aData[n+3];
            }          
          #ifndef SUPPORT_UTF8
          	// TTA----Fixed a Global bug. We will remove a character in the end, if that is '='.
            if (aData[strlength-1] == '=')
              aData[--strlength] = '\0';
            else
          #endif //SUPPORT_UTF8
              aData[strlength] = '\0';
            i--;
          }
        }
      }
    }
  }
  // else do nothing
  return;
}


vcard_element_types CVCardPhoneNumbers::getVCardElementType( char *aData)
{
  vcard_element_types vcardElementType = VCARD_NOT_A_USED_ELEMENT;
  size_t strlength = strlen( aData);
  char *elementType = NULL;

  if( strlength > 4)
  {
    // We got TEL;...;  ... VOICE CELL, we most remove the fax once
    if( aData[0] == 'T' && 
        aData[1] == 'E' &&
        aData[2] == 'L' )
    {
      // First check if it is HOME element
      elementType = strstr( aData, VCARD_STR_HOME);
      if( elementType != NULL)
      {        
        vcardElementType = VCARD_NUMBER_HOME;
        DBG( "ElementType HOME\n");
      }
      else
      {
        // Second check if it is WORK element;        
        elementType = strstr( aData, VCARD_STR_WORK);
        if( elementType != NULL)
        {
          vcardElementType = VCARD_NUMBER_WORK;
          DBG( "ElementType WORK\n");
        }
        else
        {
          // Third check if it is CELL
          elementType = strstr( aData, VCARD_STR_CELL);
          if( elementType != NULL)
          {
            vcardElementType = VCARD_NUMBER_CELL;
            DBG( "ElementType CELL\n");
          }
          else
          {
            vcardElementType = VCARD_NUMBER_DEFAULT;
            DBG( "ElementType DEFAULT\n");
          }
        }
      }      
    }
  }
    
  return vcardElementType;
}

bool CVCardPhoneNumbers::parseVCardElement( vcard_element_types aElementType, char *aData)
{
  bool returnValue = false;

  if( aElementType == VCARD_NOT_A_USED_ELEMENT)
  {
    returnValue = false;
  }
  else
  {
    returnValue = storeElement(aElementType, aData);
  }
  return returnValue;
}

bool CVCardPhoneNumbers::storeElement( vcard_element_types aElementType, char *aData)
{
  bool returnValue = false;
  unsigned int i = 0, startIndexElement = 0;
  size_t stringlength = strlen(aData);
    
  for( i = 0; i < VCARD_MAXNUMBER_LEN; i++)
    iNumber[i] = '\0';

  for( i = 0; i < stringlength; i++)
  {
    // find : because behind it there is the number/name
    if( aData[i] == ':')
    {      
      startIndexElement = i+1; // found the element
      break;
    }
  }
  
  if( startIndexElement != 0)
  {
    for(int n = startIndexElement; n < stringlength; n++)
    {
      if( aData[n] == '\r')
        aData[n] = '\0';  // remove the carriage return
      if( aData[n] == '\f')
        aData[n] = '\0';  // remove the formfeed           
    }
    
    size_t inputLength = strlen(&aData[startIndexElement]);
        
    if( inputLength > (VCARD_MAXNUMBER_LEN-1))
      inputLength = VCARD_MAXNUMBER_LEN-1;
          
    returnValue = true;
    
    switch( aElementType)
    {
      case VCARD_NUMBER_DEFAULT:
      case VCARD_NUMBER_HOME:
      case VCARD_NUMBER_WORK: 
      case VCARD_NUMBER_CELL:
        // write it to file
        strncpy( iNumber, &aData[startIndexElement], inputLength);
        break;
      default:
        returnValue = false;
        break;
    }
    if( returnValue == true)
    {
      writeNumberToFile(iNumber, aElementType);
    }
  }
  
  return returnValue;
}

void CVCardPhoneNumbers::writeNumberToFile(char *phoneNumber, vcard_element_types aElementType)
{
  char vcardElementName[20];
  FILE* fp = fopen( iFilename, "a");
  if (fp == NULL) 
  {
    DBG("Unable to openfile (%s) for writing\n", phoneNumber);
    return;
  }

  if( phoneNumber != NULL)
  {                           
    switch( aElementType)
    {
      case VCARD_NUMBER_DEFAULT:
        strcpy( vcardElementName, "DEFAULT");
        break;
      case VCARD_NUMBER_HOME:
        strcpy( vcardElementName, "HOME");        
        break;
      case VCARD_NUMBER_WORK:
        strcpy( vcardElementName, "WORK"); 
        break;
      case VCARD_NUMBER_CELL:
        strcpy( vcardElementName, "CELL"); 
        break;
      default:
        strcpy( vcardElementName, "DEFAULT"); 
        break;      
    }

    fprintf( fp, "\"%s\",\"%s\",\"%s\"\n", iName, phoneNumber, vcardElementName);  
  }

  fclose( fp );
}

void CVCardPhoneNumbers::parseVCard( char *aData)
{ 
  char *token;
  char delimit[] = "\n"; 
  char *next_token = NULL;
  char *name = NULL;

  if( aData == NULL)
  {
    DBG("Empty vcard, no data\n");
  }
  else
  {  
    name = strstr( aData, "\nN:");
    if( name == NULL)
    {
      name = strstr( aData, "\nN;");
    }
    
    // when name equal NULL there is no name in the vcard
    // this is posible! 
    if( name != NULL)
    {
      setName( name);
    }
    else
    {
      DBG("no NAME\n");
    }
  
    token = strtok(aData, delimit);  
   
    // While there are tokens in aData
    while (token != NULL)
    {
      // Get next token:
      if (token != NULL)
      {
          bool returnValue = parseVCardElement( getVCardElementType( token ), token);          
          token = strtok(NULL, delimit);
      }
    }  
  }
}

void CVCardPhoneNumbers::dumpElements(void)
{
  if( iName[0] != '\0')
  {
    DBG("Name: %s\n", iName);
  }
  else
  {
    DBG("Name: -NOT FOUND-\n");
  }
  
  if( iFilename[0] != '\0')
  {
    DBG("Filename: %s\n", iFilename);
  }
  else
  {
    DBG("Filename: -NOT FOUND-\n");
  }
}

