
#!/bin/sh -e

case "$1" in
at4x0a)
	echo "$0: not yet supported \"$1\"."
	exit 1
	PLATFORM="at4x0a"
	CROSS_BUILD="i686-pc-linux-gnu"
	CROSS_TARGET="arm-${PLATFORM}-linux-gnueabi"
	CROSS_OPT="-O2"
	CROSS_CPU="arm926ej-s"
	;;
""|s3c24xx)
	PLATFORM="s3c24xx"
	CROSS_BUILD="i686-linux"
	CROSS_TARGET="arm-linux"
	CROSS_OPT="-O2"
	CROSS_CPU="arm920"
	;;
s3c64xx)
	PLATFORM="s3c64xx"
	CROSS_BUILD="i686-pc-linux-gnu"
	CROSS_TARGET="arm-none-linux-gnueabi"
  CROSS_OPT="-O2 -mcpu=arm1176jzf-s -mfpu=vfp -mfp=vfp -march=armv6k -mtune=arm1176jzf-s -mfloat-abi=softfp"
	CROSS_CPU="arm1176jzf-s"
	;;
omap2)
	echo "$0: not yet supported \"$1\"."
	exit 1
	PLATFORM="omap2"
	CROSS_BUILD="i686-pc-linux-gnu"
	CROSS_TARGET="arm-${PLATFORM}-linux-gnueabi"
	CROSS_OPT="-O2"
	CROSS_CPU="arm1136jf-s"
	;;
titan2)
	PLATFORM="titan2"
	CROSS_BUILD="i686-pc-linux-gnu"
	#CROSS_TARGET="arm-${PLATFORM}-linux-gnueabi"
	# Build for now with temp toolchain 
	CROSS_TARGET="arm-none-linux-gnueabi"
	CROSS_OPT="-O2"
	CROSS_CPU="arm1136jf-s"
	;;
*)
	echo "$0: ERROR: Unknown platform \"$1\"."
	exit 1
	;;
esac

# now build syncml
PROGNAME="syncml"
SRCDIR="${PWD}"

OBJDIR="${PWD}/obj-${PLATFORM}"
INSDIR="${PWD}/ins-${PLATFORM}"

export BUILD_CC="gcc"
export CC="${CROSS_TARGET}-gcc"
export CFLAGS="${CROSS_OPT} -mcpu=${CROSS_CPU} -ffunction-sections -fdata-sections"
export CXX="${CROSS_TARGET}-g++"
export CXXFLAGS="${CFLAGS}"
export F77="${CROSS_TARGET}-g77"
export AS="${CROSS_TARGET}-as"
export LD="${CROSS_TARGET}-ld"
export LDFLAGS="-Wl,--gc-sections"
export AR="${CROSS_TARGET}-ar"
export NM="${CROSS_TARGET}-nm"
export RANLIB="${CROSS_TARGET}-ranlib"
export STRIP="${CROSS_TARGET}-strip"

export THIRDPARTY_BASE_DIR="`pwd`/../.."

export TTMP_LIBS="-L${THIRDPARTY_BASE_DIR}/syncml/tomtom/libs/${PLATFORM} -lttmp"
export LIBOPENOBEX_LIBS="-L${THIRDPARTY_BASE_DIR}/openobex/dist/1.3/Linux/${PLATFORM}/lib -lopenobex"
export LIBXML_LIBS="-L${THIRDPARTY_BASE_DIR}/libxml2/dist/2.6.20/Linux/lib-${PLATFORM} -lxml2"
export ZLIB="-L${THIRDPARTY_BASE_DIR}/zlib/dist/1.2.3/Linux/lib-${PLATFORM} -lz"
export WBXML2="-L${THIRDPARTY_BASE_DIR}/wbxml2/dist/0.9.2/Linux/lib-${PLATFORM} -lwbxml2"
export SYNCMLLIB="-L${THIRDPARTY_BASE_DIR}/libsyncml/dist/0.4.1/Linux/lib-${PLATFORM} -lsyncml"
export GLIB="-L${THIRDPARTY_BASE_DIR}/glib/dist/2.10.3/Linux/${PLATFORM} -lglib-2.0"
export LIBGTHREAD="-L${THIRDPARTY_BASE_DIR}/glib/dist/2.10.3/Linux/${PLATFORM} -lgthread-2.0"

export TTMP_LIBS_CFLAGS="-I${THIRDPARTY_BASE_DIR}/syncml/tomtom/includes"
export LIBOPENOBEX_CFLAGS="-I${THIRDPARTY_BASE_DIR}/openobex/dist/1.3/Include \
                           -I${THIRDPARTY_BASE_DIR}/openobex/dist/1.3/Include/openobex"
export LIBXML_CFLAGS="-I${THIRDPARTY_BASE_DIR}/libxml2/dist/2.6.20/Include/libxml2/libxml"
export SYNCMLLIB_CFLAGS="-I${THIRDPARTY_BASE_DIR}/libsyncml/dist/0.4.1/Include \
                         -I${THIRDPARTY_BASE_DIR}/libsyncml/dist/0.4.1/Include/libsyncml-1.0 \
                         -I${THIRDPARTY_BASE_DIR}/libsyncml/dist/0.4.1/Include/libsyncml-1.0/libsyncml"
export GLIB_CFLAGS="-I${THIRDPARTY_BASE_DIR}/glib/dist/2.10.3/Include \
                    -I${THIRDPARTY_BASE_DIR}/glib/dist/2.10.3/Include/glib-2.0 \
                    -I${THIRDPARTY_BASE_DIR}/glib/dist/2.10.3/Linux/${PLATFORM}/glib-2.0/include"

# Let the following depend on the platform 
case "${PLATFORM}" in
s3c64xx|titan2)
   export LIBBLUETOOTH_CFLAGS="-I${THIRDPARTY_BASE_DIR}/bluez/dist/3.24/Include"
   export LIBBLUETOOTH_LIBS="-L${THIRDPARTY_BASE_DIR}/bluez/dist/3.24/Linux/lib-${PLATFORM} -lbluetooth"
   ;;
*)
   export LIBBLUETOOTH_CFLAGS="-I${THIRDPARTY_BASE_DIR}/bluez/dist/libs-2.15/Include"
   export LIBBLUETOOTH_LIBS="-L${THIRDPARTY_BASE_DIR}/bluez/dist/libs-2.15/Linux/lib-${PLATFORM} -lbluetooth"
   ;;   
esac

if test "${PLATFORM}" = "s3c64xx" ; then
   export EXPAT="-L${THIRDPARTY_BASE_DIR}/expat/dist/2.0.1/Linux/lib-${PLATFORM} -lexpat"
   export EXPAT_CFLAGS="-I${THIRDPARTY_BASE_DIR}/expat/dist/2.0.1/Include"
else
   export EXPAT="-L${THIRDPARTY_BASE_DIR}/expat/dist/1.95.8/Linux/lib-${PLATFORM} -lexpat"
   export EXPAT_CFLAGS="-I${THIRDPARTY_BASE_DIR}/expat/dist/1.95.8/Include"
fi 


export CFLAGS="${CFLAGS} ${LIBOPENOBEX_CFLAGS} ${LIBXML_CFLAGS} ${SYNCMLLIB_CFLAGS} ${GLIB_CFLAGS} ${LIBBLUETOOTH_CFLAGS} ${EXPAT_CFLAGS}"
export LDFLAGS="${LDFLAGS} ${TTMP_LIBS} ${SYNCMLLIB} ${GLIB} ${LIBOPENOBEX_LIBS} ${LIBGTHREAD} ${LIBXML_LIBS} ${WBXML2} ${ZLIB} ${EXPAT} ${LIBBLUETOOTH_LIBS}"

make clean
echo "**** Building ${PROGNAME} for ${PLATFORM}..."
make 

# echo "**** Installing ${PROGNAME} for ${PLATFORM}..."
#make DESTDIR="${INSDIR}" install
